@@ -1,4 +1,5 @@
 import fnmatch
+import functools
 import logging
 import os
 import re
@@ -57,6 +58,7 @@ from atopile.version import (
     get_installed_atopile_version,
 )
 from faebryk.libs.exceptions import UserResourceException
+from faebryk.libs.paths import get_config_dir
 from faebryk.libs.test.testutil import in_test
 from faebryk.libs.util import indented_container, md_list
 
@@ -150,22 +152,15 @@ class ConfigFileSettingsSource(YamlConfigSettingsSource, ABC):
 class GlobalConfigSettingsSource(ConfigFileSettingsSource):
     @classmethod
     def find_config_file(cls) -> Path | None:
-        """Find the global config file in the user's home directory."""
-
-        # note deliberate use of ~/.config on all platforms
-        # (rather than e.g. platformdirs)
-        config_dir = (
-            Path(os.environ.get("XDG_CONFIG_HOME", Path.home() / ".config"))
-            / APPLICATION_NAME
-        )
-        config_file = config_dir / GLOBAL_CONFIG_FILENAME
+        config_file = get_config_dir() / GLOBAL_CONFIG_FILENAME
         return config_file if config_file.exists() else None
 
     def get_data(self) -> dict[str, Any]:
         return self.yaml_data if self.yaml_data else {}
 
 
-def _find_project_dir(start: Path) -> Path | None:
+@functools.cache
+def find_project_dir(start: Path) -> Path | None:
     """
     Search parent directories, up to the root, for a directory containing a project
     config file.
@@ -179,9 +174,9 @@ def _find_project_dir(start: Path) -> Path | None:
     return path.resolve().absolute()
 
 
-def _find_project_config_file(start: Path) -> Path | None:
+def find_project_config_file(start: Path) -> Path | None:
     """Search parent directories, up to the root, for a project config file."""
-    if (project_dir := _find_project_dir(start)) is not None:
+    if (project_dir := find_project_dir(start)) is not None:
         return project_dir / PROJECT_CONFIG_FILENAME
 
     return None
@@ -196,7 +191,7 @@ class ProjectConfigSettingsSource(ConfigFileSettingsSource):
         if _project_dir:
             return _project_dir / PROJECT_CONFIG_FILENAME
 
-        return _find_project_config_file(Path.cwd())
+        return find_project_config_file(Path.cwd())
 
     def get_data(self) -> dict[str, Any]:
         return self.yaml_data or {}
@@ -394,10 +389,10 @@ class BuildTargetConfig(BaseConfigModel, validate_assignment=True):
     Everything that exists within this module will be built as part of this build-target
     """
 
-    targets: list[str] = Field(default=["__default__"])  # TODO: validate
+    targets: list[str] = Field(default_factory=list)
     """A list of targets' names to build after updating the layout"""
 
-    exclude_targets: list[str] = Field(default=[])
+    exclude_targets: list[str] = Field(default_factory=list)
     """
     A list of targets' names to exclude.
 
@@ -406,7 +401,7 @@ class BuildTargetConfig(BaseConfigModel, validate_assignment=True):
     which typically builds **all** targets.
     """
 
-    exclude_checks: list[str] = Field(default=[])
+    exclude_checks: list[str] = Field(default_factory=list)
     """
     A list of checks to exclude.
 
@@ -755,6 +750,9 @@ class ProjectConfig(BaseConfigModel):
     open_layout_on_build: bool = Field(default=False)
     """Automatically open pcbnew when applying netlist"""
 
+    dangerously_skip_ssl_verification: bool = Field(default=True)  # FIXME: SSL
+    """Skip SSL verification for all API requests."""
+
     @classmethod
     def from_path(cls, path: Path | None) -> "ProjectConfig | None":
         if path is None:
@@ -970,6 +968,9 @@ class Config:
         yield "selected_builds", self._selected_builds
         yield "project_dir", self._project_dir
 
+    def reload(self):
+        self._project = _try_construct_config(ProjectSettings)
+
     @property
     def project(self) -> ProjectSettings | ProjectConfig:
         if self._project is None:
@@ -1170,7 +1171,8 @@ class Config:
         self,
         entry: str | None,
         standalone: bool = False,
-        target: Iterable[str] = (),
+        include_targets: Iterable[str] = (),
+        exclude_targets: Iterable[str] = (),
         selected_builds: Iterable[str] = (),
         frozen: bool | None = None,
         working_dir: Path | None = None,
@@ -1184,7 +1186,7 @@ class Config:
         if standalone:
             self._setup_standalone(entry, entry_arg_file_path)
         else:
-            if config_file_path := _find_project_config_file(entry_arg_file_path):
+            if config_file_path := find_project_config_file(entry_arg_file_path):
                 self.project_dir = config_file_path.parent
             elif entry is None:
                 raise UserNoProjectException(search_path=entry_arg_file_path)
@@ -1222,8 +1224,12 @@ class Config:
 
             if entry_addr_override is not None:
                 build_cfg.address = entry_addr_override
-            if target:
-                build_cfg.targets = list(target)
+
+            if include_targets:
+                build_cfg.targets = list(include_targets)
+
+            if exclude_targets:
+                build_cfg.exclude_targets = list(exclude_targets)
 
             # Attach CLI options passed via kwargs
             for key, value in kwargs.items():
