@@ -47,3 +47,25 @@ class BJT(Module):
             accept_prefix=False,
             case_sensitive=False,
         )
+
+    usage_example = L.f_field(F.has_usage_example)(
+        example="""
+        import BJT, Resistor, ElectricPower
+
+        bjt = new BJT
+        bjt.doping_type ="NPN"
+        bjt.mpn = "C373737
+
+        # Use as amplifier with bias resistors
+        base_resistor = new Resistor
+        collector_resistor = new Resistor
+        power_supply = new ElectricPower
+
+        # Basic amplifier configuration
+        power_supply.hv ~> collector_resistor ~> bjt.collector
+        bjt.emitter ~ power_supply.lv
+        input_signal ~> base_resistor ~> bjt.base
+        output_signal ~ bjt.collector
+        """,
+        language=F.has_usage_example.Language.ato,
+    )
