@@ -25,3 +25,13 @@ class PDM(ModuleInterface):
         return F.has_single_electric_reference_defined(
             F.ElectricLogic.connect_all_module_references(self)
         )
+
+    def __postinit__(self, *args, **kwargs):
+        super().__postinit__(*args, **kwargs)
+        self.data.line.add(F.has_net_name("DATA", level=F.has_net_name.Level.SUGGESTED))
+        self.clock.line.add(
+            F.has_net_name("CLOCK", level=F.has_net_name.Level.SUGGESTED)
+        )
+        self.select.line.add(
+            F.has_net_name("SELECT", level=F.has_net_name.Level.SUGGESTED)
+        )
